searchNodes=[{"doc":"ExPropriate is an Elixir library that allows you to decide whether or not a function is public at compile time. First you have to set up your configuration for test and any other environment you wish your functions to be expropriated in like this: # config/test.exs config ExPropriate , enable : true It may also be useful to configure your dev environment like this, so you can test expropriated functions in console. # config/dev.exs config ExPropriate , enable : System . get_env ( &quot;EXPROPRIATE_ENABLED&quot; ) == &quot;true&quot; EXPROPRIATE_ENABLED=true iex -S mix iex&gt; Once your config is setup, you can start using ExPropriate in your modules. There are two ways in which it can be used: Module level granularity via ExPropriate.FullModule Function level granularity via ExPropriate.MarkedFunctions","ref":"ExPropriate.html","title":"ExPropriate","type":"module"},{"doc":"This module handles expropriation of module-level granularity. It can be set up like this: defmodule MyModule do use ExPropriate , expropriate_all : true defp my_private_function do :has_been_expropriated end end MyModule . my_private_function # :has_been_expropriated It is important to note that this module has been designed to be less &quot; intrusive &quot; than the function-level granularity. The reason behind this decision is so that it can serve as a fallback in case there are issues with the function-level. Also a friendly reminder that the functions and macros contained in this module are for internal use of the library and it's advised against using them directly.","ref":"ExPropriate.FullModule.html","title":"ExPropriate.FullModule","type":"module"},{"doc":"An override of Kernel.defp/2 . This macro basically forwards the received args to the implicit def/2 . This usually is Kernel.def/2 , but it could be some other module.","ref":"ExPropriate.FullModule.html#defp/2","title":"ExPropriate.FullModule.defp/2","type":"macro"},{"doc":"Generates the ast necessary to inject this module's defp/2 macro instead of Kernel.defp/2 .","ref":"ExPropriate.FullModule.html#generate_use_ast/0","title":"ExPropriate.FullModule.generate_use_ast/0","type":"function"},{"doc":"This module handles expropriation of function-level granularity. It can be set up like this: defmodule MyModule do use ExPropriate # Function becomes public @expropriate true defp expropriated_function , do : :am_expropriated # Functions with multiple bodies only need to be tagged on the first body @expropriate true defp divide_by ( number ) when is_integer ( number ) and number &gt; 0 , do : { :ok , div ( 100 , number ) } defp divide_by ( _other ) , do : :error # Untagged functions remain private defp remains_private , do : :am_private end MyModule . expropriated_function # :am_expropriated MyModule . divide_by ( 2 ) # { :ok, 50 } MyModule . divide_by ( 0 ) # :error MyModule . remains_private # (UndefinedFunctionError) function MyModule.remains_private/0 is undefined or private. The objective of this module was to be able to explicitly state which functions need to be expropriated. The tradeoff is that this module is more &quot; intrusive &quot; than the module-level granularity, since it overrides both Kernel.def/2 and Kernel.defp/2 . Also a friendly reminder that the functions and macros contained in this module are for internal use of the library and it's advised against using them directly.","ref":"ExPropriate.MarkedFunctions.html","title":"ExPropriate.MarkedFunctions","type":"module"},{"doc":"An override of Kernel.def/2 . This macro checks if the @expropriate attribute was set to true before defining a public function, and outputs a warning if that's the case. Regardless of the warning, it always defines the fuction using Kernel.def/2","ref":"ExPropriate.MarkedFunctions.html#def/2","title":"ExPropriate.MarkedFunctions.def/2","type":"macro"},{"doc":"Defines a public function via Kernel.def/2 , and sets the necessary attributes for the following bodies. Called by expropriated functions when they define their first body. In addition to defining the function as public, this function also sets the @expropriate attribute back to false and registers the function's name to the @expropriated_names attribute. If the function has multiple bodies, it will directly call define_public/2 instead.","ref":"ExPropriate.MarkedFunctions.html#define_first_public_body/3","title":"ExPropriate.MarkedFunctions.define_first_public_body/3","type":"function"},{"doc":"Defines a private function via Kernel.defp/2 Called by functions that are not being expropriated.","ref":"ExPropriate.MarkedFunctions.html#define_private/2","title":"ExPropriate.MarkedFunctions.define_private/2","type":"function"},{"doc":"Defines a public function via Kernel.def/2 Called by functions that are being expropriated.","ref":"ExPropriate.MarkedFunctions.html#define_public/2","title":"ExPropriate.MarkedFunctions.define_public/2","type":"function"},{"doc":"An override of Kernel.defp/2 . This macro decides whether or not the expropriate the function body based on the following criteria: The @expropriate attribute is set to true The function's name ( fn_name/0 ) was already expropriated. (For functions with multiple bodies)","ref":"ExPropriate.MarkedFunctions.html#defp/2","title":"ExPropriate.MarkedFunctions.defp/2","type":"macro"},{"doc":"Transforms a function's head AST into a tuple containing the name and arity of the function.","ref":"ExPropriate.MarkedFunctions.html#fn_head_to_name/1","title":"ExPropriate.MarkedFunctions.fn_head_to_name/1","type":"function"},{"doc":"Generates AST to prevent warnings for unused @expropriate attributes. These warnings happened when the module has @expropriate attributes, but ExPropriate is disabled at a config level. Eg: in prod environment.","ref":"ExPropriate.MarkedFunctions.html#generate_unused_ast/0","title":"ExPropriate.MarkedFunctions.generate_unused_ast/0","type":"function"},{"doc":"Generates the AST necessary to expropriate only tagged functions on compile time. Injects this module's def/2 and defp/2 macro in favor of Kernel.def/2 and Kernel.defp/2 . Setups attributes that are later used by the def/2 and defp/2 macros.","ref":"ExPropriate.MarkedFunctions.html#generate_use_ast/0","title":"ExPropriate.MarkedFunctions.generate_use_ast/0","type":"function"},{"doc":"AST containing the definition of a function's body. Typically it's a keyword list containing at least a [do: expr] . It also may be nil (if only the function head is being declared), and it may also contain other keys like :rescue , :catch , :after , etc. [ do : { :+ , [ ] , [ 1 , 2 ] } ] |&gt; Macro . to_string # &quot;[do: 1 + 2]&quot; [ do : { :/ , [ ] , [ { :a , [ ] , nil } , { :b , [ ] , Elixir } ] } , rescue : [ { :-&gt; , [ ] , [ [ { :error , [ ] , nil } ] , { { :. , [ ] , [ { :__aliases__ , [ ] , [ :IO ] } , :puts ] } , [ ] , [ { :error , [ ] , nil } ] } ] } ] ] |&gt; Macro . to_string # &quot;[do: a / b, rescue: (error -&gt; IO.puts(error))]&quot;","ref":"ExPropriate.MarkedFunctions.html#t:fn_body/0","title":"ExPropriate.MarkedFunctions.fn_body/0","type":"type"},{"doc":"AST containing the definition of a function's head. Contains at least the function name, arguments and clauses. Does not contain def or defp . Examples : { :with_no_args , [ ] , [ ] } |&gt; Macro . to_string # &quot;with_no_args()&quot; { :with_two_args , [ ] , [ { :arg1 , [ ] , nil } , { :arg2 , [ ] , nil } } |&gt; Macro . to_string # &quot;with_two_args(arg1, arg2)&quot; { :when , [ ] , [ { :with_when , [ ] , [ { :arg , [ ] , nil } ] } , { :&gt; , [ ] , [ { :arg , [ ] , nil } , 0 ] } ] } |&gt; Macro . to_string # &quot;with_when(arg) when arg &gt; 1&quot;","ref":"ExPropriate.MarkedFunctions.html#t:fn_head/0","title":"ExPropriate.MarkedFunctions.fn_head/0","type":"type"},{"doc":"Tuple containing a function's name and arity. { :my_function , 2 }","ref":"ExPropriate.MarkedFunctions.html#t:fn_name/0","title":"ExPropriate.MarkedFunctions.fn_name/0","type":"type"},{"doc":"ExPropriate is an Elixir library that allows you to decide whether or not a function is public at compile time. When would you want to do this? Testing. There's an argument to be had that you should be testing your private functions through your public interfaces, and I agree! But it's also true that often times projects grow more complicated than you'd wish. The aim of this library is to provide a way to be able to test your overly complicated private functions without making changes in the design. When push comes to shove, a questionably designed but well tested function is better than a questionably designed and vaguely tested one.","ref":"readme.html","title":"ExPropriate","type":"extras"},{"doc":"In mix.exs , add the ExPropriate dependency: def deps do [ { :ex_propriate , &quot;~&gt; 0.1&quot; } ] end Then run mix deps.get to install it. $ mix deps.get Now add this configuration in your test config, or any other env you wish your functions to be expropriated. # config/test.exs config ExPropriate , enable : true It may also be useful to configure your dev environment like this, so you can test expropriated functions in console. # config/dev.exs config ExPropriate , enable : System . get_env ( &quot;EXPROPRIATE_ENABLED&quot; ) == &quot;true&quot; EXPROPRIATE_ENABLED=true iex -S mix iex&gt;","ref":"readme.html#installation","title":"ExPropriate - Installation","type":"extras"},{"doc":"There are two ways in which you can use ExPropriate in a module: Module level granularity Function level granularity Module Level Granularity : This level will expropriate all private functions in the module, giving them public access. defmodule MyModule do use ExPropriate , expropriate_all : true defp my_private_function , do : :has_been_expropriated end MyModule . my_private_function # :has_been_expropriated It's also worth mentioning that this granularity level is by design less &quot; intrusive &quot; than the function one, so if you're having issues with the later, you can try using this one as a fallback. Function Level Granularity : This level will only expropriate functions that are tagged with the @expropriate attribute set to true . defmodule MyModule do use ExPropriate # Function becomes public @expropriate true defp expropriated_function , do : :am_expropriated # Functions with multiple bodies only need to be tagged on the first body @expropriate true defp divide_by ( number ) when is_integer ( number ) and number &gt; 0 , do : { :ok , div ( 100 , number ) } defp divide_by ( _other ) , do : :error # Untagged functions remain private @expropriate false defp remains_private , do : :am_private end MyModule . expropriated_function # :am_expropriated MyModule . divide_by ( 2 ) # { :ok, 50 } MyModule . divide_by ( 0 ) # :error MyModule . remains_private # (UndefinedFunctionError) function MyModule.remains_private/0 is undefined or private. This graph may also help you visualize how ExPropriate decides when and which private functions to expropriate. graph TD config_enabled{{Enabled in Config?}} config_enabled -- false --&gt; everything_private[All defp remain private] config_enabled -- true --&gt; expropriate_all{{use ExPropriate, expropriate_all: ?}} expropriate_all -- true --&gt; module_becomes_public[All defp in the Module become public] expropriate_all -- false --&gt; module_remains_private[All defp in the Module remain private] expropriate_all -- nil --&gt; function_is_tagged{{&quot;@expropriate true&quot;}} function_is_tagged -- true --&gt; function_becomes_public[Tagged defp becomes public] function_is_tagged -- false --&gt; function_remains_private[Tagged defp remains private]","ref":"readme.html#usage","title":"ExPropriate - Usage","type":"extras"},{"doc":"Some libraries define their own versions of def and defp . It's advised against using ExPropriate in modules that implement such libraries. If you want to try regardless, please be aware that: Only Module Granularity ( expropriate_all ) is ( loosely ) supported. Function-level granularity explicitly defines functions with Kernel.def/2 and Kernel.defp/2 , so even if things seem to work at first, you are probably loosing the functionality that you're seeking from the other library. If the library only overrides def/2 Libraries that override Kernel.def/2 typically do so to enhance functionality. Since expropriating a function basically makes it public, be wary of any unwanted side-effect that may occur. Other than that, the setup should look something like this: defmodule MyModule do use OtherLibrary use ExPropriate , expropriate_all : true end If the library also overrides defp/2 Welp. You really shouldn't use ExPropriate in that module, but if you absolutely must... You'll be trading off the functionality of that library's defp/2 whenever you expropriate the functions. You'll also be responsible of preventing the import of said library's defp/2 , otherwise you'll get compilation errors. The setup would look something like this: demfodule MyModule do use OtherLibrary if Application . compile_env ( :ex_propriate , :enable ) do import OtherLibrary.Macros , except : [ defp : 1 , defp : 2 ] use ExPropriate , expropriate_all : true do end end","ref":"readme.html#limitations","title":"ExPropriate - Limitations","type":"extras"},{"doc":"ExPropriate is released under the MIT License .","ref":"readme.html#license","title":"ExPropriate - License","type":"extras"}]